import {
  csvParse,
  select,
  geoNaturalEarth1,
  geoPath,
  geoGraticule,
  scaleOrdinal,
} from 'd3';
import {
  colorData,
  colorScale,
  numberScale,
} from './color.js';
import { colorLegend } from './colorLegend.js';

const projection = geoNaturalEarth1();
const path = geoPath(projection);
const graticule = geoGraticule();

const tooltip = d3
  .select('body')
  .append('div')
  .attr('class', 'd3-tooltip')
  .style('position', 'absolute')
  .style('z-index', '10')
  .style('visibility', 'hidden')
  .style('padding', '10px')
  .style('background', 'rgba(0,0,0,0.6)')
  .style('border-radius', '5px')
  .style('color', 'white');

export const map = (selection, { data }) => {
  fetch('internet_users.csv')
    .then((response) => response.text())
    .then((csvString) => {
      let internet = csvParse(csvString);

      function usersPerCountry(country) {
        let users = 'No Data';
        internet.forEach((c) => {
          if (c.country_code == country) {
            users = c['2015'];
          }
        });
        let round = Math.round(users * 100) / 100;
        if (isNaN(round)) {
          return users;
        }
        else return round;
          
      }

      function getColor(num) {
        let color = colorData(num);
        return color;
      }

      selection
        .selectAll('path.graticule')
        .data([null])
        .join('path')
        .attr('class', 'graticule')
        .attr('d', path(graticule()))
        .attr('fill', 'none')
        .attr('stroke', '#BBB')
        .attr('stroke-width', 0.5);

      selection
        .selectAll('path.outline')
        .data([null])
        .join('path')
        .attr('class', 'outline')
        .attr('d', path(graticule.outline()))
        .attr('fill', '#d6ebff')
        .attr('stroke', 'black')
        .attr('stroke-width', 2);

      selection
        .selectAll('path.country')
        .data(data.features)
        .join('path')
        .attr('d', path)
        .attr('class', 'country')
        .attr('stroke', 'black')
        .attr('fill', (d) =>
          getColor(
            usersPerCountry(d.properties.a3)
          )
        )
        .attr('stroke-width', 0.5)
        .on('mouseover', function (e, d) {
        let count = usersPerCountry(
                d.properties.a3)
        if (count != 'No Data') {
        count = count + '%'}
          tooltip
            .html(
              `${
                d.properties.name
              }: ${count}`
            )
            .style('visibility', 'visible');
        })
        .on('mousemove', function () {
          tooltip
            .style('top', event.pageY - 10 + 'px')
            .style(
              'left',
              event.pageX + 10 + 'px'
            );
        })
        .on('mouseout', function () {
          tooltip.style('visibility', 'hidden');
        });

      const colors = colorScale();
      const range = numberScale();

      const scale = scaleOrdinal()
        .domain(range)
        .range(colors);

      selection.call(colorLegend, {
        colorScale: scale,
        colorLegendLabel:
          'Internet Users Per Country (2015)',
        colorLegendX: 75,
        colorLegendY: 300,
      });

      const zoom = d3
        .zoom()
        .scaleExtent([1, 10])
        .on('zoom', zoomed);

      selection.call(zoom);

      function zoomed(event) {
        const { transform } = event;
        console.log(transform)
        selection.attr('transform', transform);
        selection.attr(
          'stroke-width',
          1 / transform.k
        );
      }
    });
};
