import { csvParse, select } from 'd3';
import topojson from 'topojson-client';
import { map } from './map';

const worldAtlasURL =
  'https://unpkg.com/visionscarto-world-atlas@0.1.0/world/110m.json';

export const viz = (
  container,
  { state, setState }
) => {
  const width = window.innerWidth;
  const height = window.innerHeight;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  // state.data could be:
  // * undefined
  // * 'LOADING'
  // * An array of objects
  const { data } = state;

  if (data && data !== 'LOADING') {
    svg.call(map, {
      data,
    });
  }

  if (data === undefined) {
    setState((state) => ({
      ...state,
      data: 'LOADING',
    }));
    fetch(worldAtlasURL)
      .then((response) => response.json())
      .then((topoJSONData) => {
        // d3.csv("internet_users.csv")
        //   .then(function(csv) {
        const data = topojson.feature(
          topoJSONData,
          'countries'
        );
        // });
        setState((state) => ({
          ...state,
          data,
        }));
      });
  }
};
